"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug_1 = __importDefault(require("debug"));
const superagent_1 = __importDefault(require("superagent"));
const deep_object_diff_1 = require("deep-object-diff");
const requests_1 = require("./requests");
const log = debug_1.default('middleware:session');
const TWO_WEEKS = 1000 * 60 * 60 * 24 * 14;
var Access;
(function (Access) {
    Access["OWNER"] = "OWNER";
    Access["SIEGE"] = "SIEGE";
})(Access = exports.Access || (exports.Access = {}));
var AuthorTypes;
(function (AuthorTypes) {
    AuthorTypes["SUBMISSION"] = "submission";
    AuthorTypes["REDDIT"] = "reddit";
    AuthorTypes["GITHUB"] = "github";
    AuthorTypes["YOUTUBE"] = "youtube";
    AuthorTypes["TWITCH"] = "twitch";
})(AuthorTypes = exports.AuthorTypes || (exports.AuthorTypes = {}));
const defaultOptions = {
    url: 'localhost:3100',
    domain: 'sr.dev',
};
function Session(optionsIn) {
    const options = Object.assign({}, defaultOptions, optionsIn);
    const doPrintDebug = /middleware:session/.test(process.env.DEBUG || '');
    const print = (str) => doPrintDebug && console.log(str);
    superagent_1.default
        .get(`${options.url}`)
        .on('error', () => {
        throw new Error("Couldn't reach auth service.");
    })
        .then(response => {
        if (response.text !== 'What are you doing here?') {
            throw new Error(`Auth service responded incorrectly (${response.text}).`);
        }
        print('Auth service responded correctly');
    });
    return async function Middleware(ctx, next) {
        const sid = ctx.cookies.get('sid');
        let session;
        if (sid) {
            log('Found a SID');
            session = await requests_1.getSession(options.url, sid);
        }
        else {
            log("Couldn't find a SID");
            session = await requests_1.createSession(options.url);
        }
        ctx.session = Object.assign({}, session.session, { id: session.id, maxAge: session.maxAge, user: session.user, isLoggedIn: !!session.user, identifier: session.user ? session.user.uuid : session.id });
        ctx.cookies.set('sid', session.id, {
            maxAge: TWO_WEEKS,
            domain: '.' + options.domain,
        });
        await next();
        const _a = ctx.session, { id, maxAge, user, isLoggedIn, identifier } = _a, rest = __rest(_a, ["id", "maxAge", "user", "isLoggedIn", "identifier"]);
        const ctxSession = { id, maxAge, user, session: rest };
        const difference = deep_object_diff_1.diff(session, ctxSession);
        if (Object.keys(difference).length > 0) {
            log('Found difference in session!');
            log(difference);
            await requests_1.updateSession(options.url, id, deep_object_diff_1.diff(session, ctxSession));
        }
    };
}
exports.Session = Session;
//# sourceMappingURL=index.js.map