"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug_1 = __importDefault(require("debug"));
const superagent_1 = __importDefault(require("superagent"));
const log = debug_1.default('middleware:session:requests');
async function getSession(url, sid) {
    let response;
    try {
        log(`Trying to get SID:${sid}`);
        response = await superagent_1.default.get(`${url}/api/session/${sid}`);
        log(`Successfully got SID:${sid}`);
    }
    catch (err) {
        response = { body: null, status: err.status || 500 };
    }
    if (response.status === 404) {
        log(`Couldn't find SID:${sid}`);
        return createSession(url);
    }
    return response.body;
}
exports.getSession = getSession;
async function createSession(url) {
    let response;
    try {
        log('Creating a new SID');
        response = await superagent_1.default.post(`${url}/api/session`);
        log(`Successfully created new SID: ${response.body.id}`);
    }
    catch (err) {
        response = { body: null };
    }
    return response.body;
}
exports.createSession = createSession;
async function updateSession(url, sid, obj) {
    let response;
    try {
        log(`Trying to update SID:${sid}`);
        response = await superagent_1.default.patch(`${url}/api/session/${sid}`).send(obj);
        log(`Successfully updated SID:${sid}`);
    }
    catch (err) {
        response = { body: null };
    }
    return response.body;
}
exports.updateSession = updateSession;
//# sourceMappingURL=requests.js.map